<?php

namespace SM\Service;

use SM\Factory\Sm00006000ElementoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use SM\Service\FindService;

class Sm00006000ElementoService
{
    protected $ElementoFactory;
    protected $Environment;
    protected $newCode;
    protected $procedureService;
    protected $FindService;
    protected $connection;

    public function __construct(EntityManager $entityManager, Sm00006000ElementoFactory $ElementoFactory, Environment $Environment, NewCode $newCode, FindService $FindService)
    {
        $this->connection         = $entityManager->getConnection();
        $this->ElementoFactory    = $ElementoFactory;
        $this->Environment        = $Environment;
        $this->newCode            = $newCode;
        $this->FindService        = $FindService;
    }

    public function getElementos($parametros)
    {
        $filtroIdAtivo = '';
        if (isset($parametros["IDATIVO"])) {
            $filtroIdAtivo = " AND ELEMENTO.IDATIVO = '" . $parametros["IDATIVO"] . "'";
        }

        $filtroNome = '';
        if (isset($parametros["NOME"])) {
            $filtroNome = " AND ELEMENTO.NMSERVORCA LIKE '%" . $parametros["NOME"] . "%'";
        }

        $filtroTipoMedidaVariavel = '';
        if (isset($parametros["CDTPMEDIDAVARIAVEL"])) {
            $parametros["CDTPMEDIDAVARIAVEL"] = $this->FindService->formatarArray($parametros["CDTPMEDIDAVARIAVEL"]);
            $filtroTipoMedidaVariavel = " AND ELEMENTO.CDTPMEDIDAVARIAVEL IN (" . $parametros["CDTPMEDIDAVARIAVEL"] . ")";
        }

        $filtroRelacionamentoServico = '';
        if (isset($parametros["RELACSERVICO"])) {
            $parametros["RELACSERVICO"] = $this->FindService->formatarArray($parametros["RELACSERVICO"]);
            $filtroRelacionamentoServico = " INNER JOIN FCT_RELACSERVICO SERVICO ON ELEMENTO.CDSERVORCA = SERVICO.CDSERVORCA AND SERVICO.CDSERVORCARELAC IN (" . $parametros["RELACSERVICO"] . ")";
        }

        $filtroFormacaoPreco = '';
        if (isset($parametros["FORMACAOPRECO"])) {
            $filtroFormacaoPreco = " AND ELEMENTO.IDFORMACAOPRECO = '" . $parametros["FORMACAOPRECO"] . "'";
        }

        $filtroCentroCusto = '';
        if (isset($parametros["CDCENTCUST"])) {
            $parametros["CDCENTCUST"] = $this->FindService->formatarArray($parametros["CDCENTCUST"]);
            $filtroCentroCusto = " AND ELEMENTO.CDCENTCUST IN (" . $parametros["CDCENTCUST"] . ")";
        }

        $idTipoElemento = '';
        if (isset($parametros["IDTIPOELEMENTO"])) {
            $idTipoElemento = " AND ELEMENTO.IDTIPOELEMENTO = '" . $parametros["IDTIPOELEMENTO"] . "'";
        }

        return $this->ElementoFactory->getElementos($filtroIdAtivo, $filtroNome, $filtroTipoMedidaVariavel, $filtroRelacionamentoServico, $filtroFormacaoPreco, $filtroCentroCusto, $idTipoElemento);
    }

    public function getTipoMedidaVariavel()
    {
        return $this->ElementoFactory->getTipoMedidaVariavel();
    }

    public function getRelacionamentoServico()
    {
        return $this->ElementoFactory->getRelacionamentoServico();
    }

    public function getCentroCusto()
    {
        return $this->ElementoFactory->getCentroCusto();
    }

    public function frameRelacionamentoServico($parametros)
    {
        return $this->ElementoFactory->frameRelacionamentoServico(array(
            "CDSERVORCA"          => $parametros["NR"]
        ));
    }

    public function salvarElementoAdd($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $CDSERVORCA = $this->newCode->getCodeByTableName("FCT_SERVICOORCA", $NRORG, 1, 10, true);

            if (!isset($parametros["IDELEMENTOPAI"])) {
                $parametros["IDELEMENTOPAI"] = null;
            }
        
            $this->ElementoFactory->salvarElementoAdd(array(
                "IDATIVO"               => $parametros["IDATIVO"],
                "NMSERVORCA"            => $parametros["NMSERVORCA"],
                "IDELEMENTOPAI"         => $parametros["IDELEMENTOPAI"],
                "CDTPMEDIDAVARIAVEL"    => $parametros["CDTPMEDIDAVARIAVEL"],
                "IDORDENAELEMENTO"      => $parametros["IDORDENAELEMENTO"],
                "IDFORMACAOPRECO"       => $parametros["IDFORMACAOPRECO"],
                "IDALTERAFORMPREC"      => $parametros["IDALTERAFORMPREC"],
                "DSSERVORCA"            => $parametros["DSSERVORCA"],
                "CDCENTCUST"            => $parametros["CDCENTCUST"],
                "NRORG"                 => $NRORG,
                "CDOPERINCLUSAO"        => $this->Environment->getCdOperador(),
                "CDSERVORCA"            => $CDSERVORCA,
                "IDELEMENTO"            => "G_" . $CDSERVORCA,
                "IDITEMCALCULO"         => $parametros["IDITEMCALCULO"],
                "IDTIPOCALCULO"         => $parametros["IDTIPOCALCULO"],
                "DSEXPRESSAOCALC"       => $parametros["DSEXPRESSAOCALC"],
                "VRPERCCALCULO"         => $parametros["VRPERCCALCULO"]
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarElementoEdit($parametros)
    {
// var_dump($parametros["CDSERVORCA"]);
// die;
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $this->ElementoFactory->salvarElementoEdit(array(
                "CDSERVORCA"            => $parametros["CDSERVORCA"],
                "IDATIVO"               => $parametros["IDATIVO"],
                "NMSERVORCA"            => $parametros["NMSERVORCA"],
                "IDELEMENTOPAI"         => !isset($parametros["IDELEMENTOPAI"]) ? null : ($parametros["IDELEMENTOPAI"]),
                "CDTPMEDIDAVARIAVEL"    => $parametros["CDTPMEDIDAVARIAVEL"],
                "IDORDENAELEMENTO"      => $parametros["IDORDENAELEMENTO"],
                "IDFORMACAOPRECO"       => $parametros["IDFORMACAOPRECO"],
                "IDALTERAFORMPREC"      => $parametros["IDALTERAFORMPREC"],
                "DSSERVORCA"            => $parametros["DSSERVORCA"],
                "CDCENTCUST"            => $parametros["CDCENTCUST"],
                "CDOPERULTATU"          => $this->Environment->getCdOperador(),
                "NRORGULTATU"           => $NRORG,
                "IDITEMCALCULO"         => $parametros["IDITEMCALCULO"],
                "IDTIPOCALCULO"         => $parametros["IDTIPOCALCULO"],
                "DSEXPRESSAOCALC"       => $parametros["DSEXPRESSAOCALC"],
                "VRPERCCALCULO"         => $parametros["VRPERCCALCULO"],
                "IDELEMENTOPAI2"        => 'G_'.$parametros["CDSERVORCA"],
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirElementos($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->FindService->formatarArray($parametros["NRS"]);
            $this->ElementoFactory->excluirElementos($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTipoMedida()
    {
        return $this->ElementoFactory->getTipoMedida();
    }

    public function salvarRelacionamentoServicoEdit($params)
    {
        $this->connection->beginTransaction();
       
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            
            foreach ($params["EDITEDROWS"] as $row) {
                
                $parametros = array (
                    "CDSERVORCA"     => $row["CDSERVORCA"],
                    "IDMEDIDAFIXA"   => $row["IDMEDIDAFIXA"],
                    "QTMEDIDASERV"   => $row["QTMEDIDASERV"],
                    "NRRELACSERVICO" => $row["NRRELACSERVICO"],
                    "NRORGULTATU"    => $nrorg,
                    "CDOPERULTATU"   => $operador
                );
                $this->ElementoFactory->salvarRelacionamentoServicoEdit($parametros);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarRelacionamentoServicoAdd($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrorg();
            $OPERADOR = $this->Environment->getCdOperador();
            $NRRELACSERVICO = $this->newCode->getCodeByTableName("FCT_RELACSERVICO", $NRORG, 1, 10, true);

                $parametros = array (
                    "NRRELACSERVICO"    => $NRRELACSERVICO,
                    "CDSERVORCARELAC"   => $parametros["CDSERVORCARELAC"],
                    "CDSERVORCA"        => $parametros["CDSERVORCA"],
                    "QTMEDIDASERV"      => $parametros["QTMEDIDASERV"],
                    "CDTPMEDIDASERV"    => $parametros["CDTPMEDIDASERV"],
                    "IDMEDIDAFIXA"      => $parametros["IDMEDIDAFIXA"],
                    "NRORG"             => $NRORG,
                    "CDOPERINCLUSAO"    => $OPERADOR,
                );
                $this->ElementoFactory->salvarRelacionamentoServicoAdd($parametros);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }  

    public function deletarRelacionamentoServico($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->FindService->formatarArray($parametros["NRS"]);
            $this->ElementoFactory->deletarRelacionamentoServico($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarCadastramentoAutomatico($params)
    {
        $this->connection->beginTransaction();
       
        try {
            $NRORG = $this->Environment->getNrorg();
            $OPERADOR = $this->Environment->getCdOperador();
            
            foreach ($params["CDSERVORCA"] as $key1 => $row) {
                foreach ($params["CDSERVORCARELAC"] as $key2 => $row) {
                    $NRRELACSERVICO = $this->newCode->getCodeByTableName("FCT_RELACSERVICO", $NRORG, 1, 10, true);
                    $parametros = array (
                        "NRRELACSERVICO"  => $NRRELACSERVICO,
                        "CDSERVORCARELAC" => $params["CDSERVORCARELAC"][$key2],
                        "CDSERVORCA"      => $params["CDSERVORCA"][$key1],
                        "QTMEDIDASERV"    => $params["QTMEDIDASERV"],
                        "CDTPMEDIDASERV"  => $params["CDTPMEDIDASERV"],
                        "IDMEDIDAFIXA"    => $params["IDMEDIDAFIXA"],
                        "NRORG"           => $NRORG,
                        "CDOPERINCLUSAO"  => $OPERADOR
                    );
                    $this->ElementoFactory->salvarCadastramentoAutomatico($parametros);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
